/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext.annotations;

import com.squeakysand.jsp.tagext.BodyContent;
import com.squeakysand.jsp.tagext.TagLibraryUtils;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Identifies a JSP Custom Tag handler and results in a <code>&lt;tag></code> element being added to the TLD.
 * 
 * For example:
 * 
 * <pre>
 * package mypackage;
 * 
 * import com.squeakysand.jsp.tagext.EnhancedSimpleTagSupport;
 * 
 * /**
 *  * The is the class level JavaDoc comment.
 *  *&#47;
 * &commat;JspTag
 * public class MyImportantTag extends EnhancedSimpleTagSupport {
 *     ... 
 * }
 * </pre>
 * 
 * would result in the following being added to the TLD:
 * 
 * <pre>
 * &lt;taglib>
 *     ...
 *     &lt;tag>
 *         &lt;description>The is the class level JavaDoc comment.&lt;/description>
 *         &lt;name>myImportant&lt;/name>
 *         &lt;tag-class>mypackage.MyImportantTag&lt;tag-class>
 *         &lt;body-content>empty&lt;body-content>
 *         ...
 *     &lt;/tag>
 *     ...
 * &lt/taglib>
 * </pre>
 * 
 * If the class <code>mypackage.MyImportantTagExtraInfo</code> also existed, then the appropriate <code>&lt;tei-class></code> element would also have been
 * added.
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.TYPE)
public @interface JspTag {

    /**
     * The value of the <code>&lt;body-content></code> element for this Tag handler in the TLD.
     */
    BodyContent bodyContent() default BodyContent.EMPTY;

    /**
     * The value for the <code>&lt;dynamic-attributes></code> element for this Tag handler in the TLD.
     */
    boolean dynamicAttributes() default false;

    /**
     * The value for the <code>&lt;example></code> element for this Tag handler in the TLD.
     */
    String example() default "";

    /**
     * The value for the <code>&lt;name></code> element for this Tag handler in the TLD. If no value is specified, a default name will be generated.
     * 
     * @see TagLibraryUtils#generateDefaultTagName(String)
     */
    String name() default "";

    /**
     * The value for the <code>&lt;tei-class></code> element for this Tag handler in the TLD. This attribute is unnecessary if the recommended TEI class naming
     * guidelines from the JSP Specification are followed as the TEI class will be discovered automatically.
     */
    String teiClass() default "";

    // static final class NO_TEI_SPECIFIED {}

}
