/*
 * Copyright 2010-2012 Craig S. Dickson (http://craigsdickson.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.squeakysand.jsp.tagext.annotations;

import com.squeakysand.jsp.tagext.VariableScope;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Inherited;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Adds a <code>&lt;variable></code> sub-element to a <code>&lt;tag></code> element in a TLD.
 * 
 * <p>
 * When applied to a static constant field, the value of the field is used as the value for the <name-given> element.
 * </p>
 * 
 * <p>
 * For example:
 * </p>
 * 
 * <pre>
 * &commat;JspTagVariable
 * public static final String MY_VARIABLE_NAME = &quot;myVariable&quot;;
 * </pre>
 * 
 * would result in the following being added to the TLD:
 * 
 * <pre>
 * &lt;taglib>
 *     ...
 *     &lt;tag>
 *         ...
 *         &lt;variable>
 *             &lt;name-given>myVariable&lt;/name-given>
 *             ...
 *         &lt;/variable>
 *         ...
 *     &lt;/tag>
 *     ...
 * &lt;/taglib>
 * </pre>
 * 
 * by declaring the variable using a constant, you can then use the same constant in your own code to actually set the value of variable, e.g.:
 * 
 * <pre>
 * ...
 * setPageAttribute(MY_VARIABLE_NAME, value);
 * ...
 * </pre>
 * 
 * <p>
 * When applied to a JavaBean setter method, the JavaBean property name derived from the method name is used as the value for the
 * <code>&lt;name-from-attribute></code> element.
 * </p>
 * 
 * <p>
 * For example:
 * </p>
 * 
 * <pre>
 * &commat;JspTagVariable
 * &commat;JspTagAttribute(required = true)
 * public void setVar(String name) {
 *     ...
 * }
 * </pre>
 * 
 * would result in the following being added to the TLD:
 * 
 * <pre>
 * &lt;taglib>
 *     ...
 *     &lt;tag>
 *         ...
 *         &lt;variable>
 *             &lt;name-from-attribute>var&lt;/name-from-attribute>
 *             ...
 *         &lt;/variable>
 *         ...
 *     &lt;/tag>
 *     ...
 * &lt;/taglib>
 * </pre>
 * 
 * 
 * @author <a href="http://craigsdickson.com">Craig S. Dickson</a>
 */
@Documented
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.FIELD, ElementType.METHOD})
public @interface JspTagVariable {

    /**
     * The value for the <code>&lt;declare></code> element.
     */
    boolean declare() default false;

    /**
     * The value for the <code>&lt;scope></code> element.
     */
    VariableScope scope() default VariableScope.NESTED;

    /**
     * The type of the variable that will be injected into the JSP page context. In the TLD, this is indicated by the <code>&lt;variable-class></code> element.
     * This attribute should be specified if the type is anything other than {@link String}.
     */
    Class<?> variableClass() default String.class;

}
